/* --------------------------------------------------------------------------
$Id: hlhdf.h,v 1.1.1.1 2001/12/14 12:02:39 ahenja Exp $
Program:        

Description:    Common functions for working with an HDF5 file.

Author(s):      Anders Henja

Copyright:	Swedish Meteorological and Hydrological Institute, 2000

History:
$Log: hlhdf.h,v $
Revision 1.1.1.1  2001/12/14 12:02:39  ahenja
First CVS version of hlhdf

Revision 1.10  2000/04/11 10:07:50  ahenja
Added function for checking wether a file is hdf5 format or not.

Revision 1.9  2000/03/27 11:35:03  ahenja
Changed my mind about strings.h and string.h, just use string.h
I don't use any X/Open specifics in my code anyway.

Revision 1.8  2000/03/27 11:28:35  ahenja
Added support for WIN32 strings, i.e. #include <string.h> instead
of #include <strings.h>

Revision 1.7  2000/03/22 08:35:25  ahenja
Changed description.

Revision 1.6  2000/03/22 08:31:55  ahenja
Updated with spelling corrections and header information.

-----------------------------------------------------------------------------*/
#ifndef HLHDF_H
#define HLHDF_H

#ifndef FALSE
#define FALSE 0
#endif

#ifndef TRUE
#define TRUE (!FALSE)
#endif

/* Don't think it is nessecary to use strings.h, only
 * use string.h instead.
 *#if defined(WIN32)
 *#include <string.h>
 *#else
 *#include <strings.h>
 *#endif
 */
#include <string.h>

#include <stdlib.h>
#include <hdf5.h>

/* Variable for beeing able to debug */
extern int _debug_hdf;

/*#define DISPLAY_INDENT 2*/
#define ALIGN(A,Z)	((((A)+(Z)-1)/(Z))*(Z))
#define DEFAULT_NOOF_CHILDS 10

typedef struct CompoundAttributeDef_t
{
   char* name;   	/*Name of attribute type, might be null,
			 *if the type not is compound*/

   size_t attrSize;	/*Size of attribute type*/

   char* format_type;	/*What type it is represented as*/
   char* ieee_type;	/*The HDF representation*/

   int ndims;		/*An attribute might be of some dimension at most 4*/
   hsize_t dims[4];	/*The dims are stored here*/
   size_t offset;

   int isString;	/*If the attribute is a string or not*/
   char* strpad;	/*The String pad type*/
   char* strcset;	/*The String character set*/
   char* strctype;	/*The String character type*/
} CompoundAttributeDef_t;

typedef struct NameListType_t {
   /*In case the type not is of compound type*/
   char* format_type;	/*What type it is represented as*/
   char* ieee_type;	/*The HDF representation*/
   int   isString;	/*If the attribute is a string or not*/
   char* strpad;	/*The String pad type*/
   char* strcset;	/*The String character set*/
   char* strctype;	/*The String character type*/
   size_t typeSize;	/*How big is the datasize*/

   /*This is used when the attribute is of a compound type*/
   int isCompound;	/*If this type is a self defined or std*/
   int isNamed;		/*If the type is compound, is it named*/
   char* name;		/*Name of the type*/

   /*This is used to indicate that there is a group owner of this type*/
   int isOwnedByGroup;

   int noAttrs;		/*Number of attribute definitions*/
   int noallocAttrs;	/*Number of attribute defs in the type*/
   CompoundAttributeDef_t** attrs;	/*The attribute defs*/

   unsigned long objno[2];	/*The object reference no, if it is compound*/
   hid_t referenceId;		/*Contains the actual reference id to the datatype as def. from HDF,
				 *Currently only used internally, do not access it from outside*/
} NameListType_t;

typedef struct NameListAttribute_t {
   char* name;			/*HDF name of this attribute*/
   int ndims;			/*How many dimensions does this attribute have*/
   hsize_t* dims;		/*What are the dimensions*/
   unsigned char* data;		/*The data set*/
   NameListType_t* attrtype;	/*How is this attribute defined*/
} NameListAttribute_t;

typedef struct NameListDataset_t {
   char* name;			/*Name of the dataset*/
   int ndims;			/*Rank of dataset*/
   hsize_t* dims;		/*The dimensions of the dataset*/
   unsigned char* data;		/*The actual data*/
   NameListType_t* datatype;	/*Describes what how the data is represented*/

   int noAttrs;			/*Number of attributes*/
   int noallocAttrs;		/*Number of allocated attributes*/
   NameListAttribute_t** attrs;	/*The attribute list*/	

   struct NameListGroup_t* rootGroup;	/*Pointer to the root group, for internal use*/
} NameListDataset_t;

typedef struct NameListGroup_t {
   char* name;		/*Name of the group*/

   /*List of child groups*/
   int noGroups;
   int noallocGroups;
   struct NameListGroup_t* groups;

   /*Keep track of the datasets*/
   int noDatasets;
   int noallocDatasets;
   NameListDataset_t* datasets;

   /*The attributes belonging to this group*/
   int noAttrs;
   int noallocAttrs;
   NameListAttribute_t** attrs;

   /*
    *Global type definitions, should only be located in the rootGroup,
    *these type definitions will then be pointed to by the
    *groupTypes
    **/
   int noGlobalTypes;
   int noallocGlobalTypes;
   NameListType_t** globalTypes;

   /*Points out this groups type definitions*/
   int noLocalTypes;
   int noallocLocalTypes;
   NameListType_t** localTypes;

   struct NameListGroup_t* rootGroup;	/*Pointer to the root group*/
   int thisIsRootGrp;			/*Indicates if this is the root group or not*/
} NameListGroup_t;

/* Activates/Deactivates HDF5 debugging */
void disableErrorReporting();
void enableErrorReporting();

/* Function: initHlHdf
 * Description:
 * 	Initiate the HLHDF handler functions,
 *	always call this before doing anything else.
 */
void initHlHdf();

/* Function: debugHlHdf
 * Description: Sets the debug mode,
 * Input:
 *       flag, 0 == No Debugging
 *             1 == Debug only the hlhdflib
 *             2 == Debug everything, including HDF library
 */
void debugHlHdf(int flag);

/* Function: isHdf5File
 * Description: Checks wether the specified file is a HDF5 file or not
 * Input:
 *	filename: The name of the file to check
 * Returns: 0 if file not is an HDF file, 1 if it is.
 */
int isHdf5File(const char* filename);

/* Function: openHlHdfFile
 * Description: Opens a HDF5 file.
 * Input:
 *       filename: The name of the HDF5 file.
 *       how: Modes can be 'r', 'w' or 'rw'
 * Returns: A hid_t reference
 */
hid_t openHlHdfFile(const char* filename,const char* how);

/* Function: createHlHdfFile
 * Description: Creates an HDF5 file, if the filename already exists,
 *		this file will be truncated
 * Input:
 *	 filename: The name of the file to create
 * Returns: A hid_t reference
 */
hid_t createHlHdfFile(const char* filename);

/* Function: closeHlHdfFile
 * Description: Closes an HDF5 file.
 * Input: file_id, A hid_t file id
 * Returns: The errors as specified in H5Fclose()
 */
herr_t closeHlHdfFile(hid_t file_id);

/* Function:	getFixedType
 * Description:	Returns a native datatype.
 * Input:
 *		hid_t	The type that should be checked
 * Returns:	<0 if failure, otherwise a reference to the new type
 * Comment:	NOTE! Call H5Tclose on the returned type.
 */
hid_t getFixedType(hid_t type);

/* Function:	translateCharToDatatype
 * Description:	Returns a native datatype from the type as
 *		specified by a string.
 * Input:
 *		dataType	The type that should be returned,
 *				can be one of:
 *				'char', 'schar', 'uchar', 'short', 'ushort',
 *				'int', 'uint', 'long', 'ulong', 'llong',
 *				'ullong', 'float', 'double', 'hsize', 'hssize',
 *				'herr', 'hbool'
 * Returns:	<0 if failure, otherwise a reference to the new type
 * Comment:	NOTE! Call H5Tclose on the returned type.
 */
hid_t translateCharToDatatype(const char* dataType);

/*Returns the type name as represented by HDF*/
/*After usage: call free() on the returned string*/
char* getTypeNameString(hid_t type);

/*Returns the format name as represented by HLHDF*/
/*After usage: call free() on the returned string*/
char* getFormatNameString(hid_t type);

/*Returns the String Pad name as represented by HDF*/
/*After usage: call free() on the returned string*/
char* getStringPadName(hid_t type);

/*Returns the CSET name as represented by HDF*/
/*After usage: call free() on the returned string*/
char* getStringCsetName(hid_t type);

/*Returns the CTYPE name as represented by HDF*/
/*After usage: call free() on the returned string*/
char* getStringCtypeName(hid_t type);

/* Function: newGroup
 * Description: Creates a new group and attaches this group to the
 *              parentGroup, unless parentGroup is NULL, which
 *              will set the returned group to the rootGroup.
 *
 * Input: parentGroup (Identifies the parent to this group)
 *                     if it is NULL, then the created group
 *                     is the root group
 *        name Name of the created group
 * Returns: Pointer to the created group
 *
 * Error handling: Hopefully this will never fail, if it does,
 *                 then I can have implemented it wrong, someone
 *                 have messed with the counters or
 *                 there is a memory allocation failiure.
 */
NameListGroup_t* newGroup(NameListGroup_t* parentGroup,const char* name);

/* Function: newDataset
 * Description: Creates a dataset and attaches this dataset to the
 *              parentGroup.
 * Input:
 *	parentGroup Identifies the group that this dataset should
 *		    be attached to
 *      name	    Name of the dataset
 * Returns: Pointer to the created dataset
 *
 * Error handling: Will return NULL if memory failure, or if the parentGroup is NULL.
 */
NameListDataset_t* newDataset(NameListGroup_t* parentGroup, const char* name);

/* Function: newNameListType
 * Description: Allocates a new NameListType_t object
 * Input: None
 * Returns: An allocated object of type NameListType_t unless
 *          there were a memory allocation failiure
 */
NameListType_t* newNameListType();

/* Function: newAttribute
 * Description: Allocates a new NameListAttribute_t object
 * Input:
 *	 name: Name of the attribute, may be NULL, then the attribute will not be given
 *             a name.
 * Returns: An allocated object of type NameListAttribute_t unless
 *          there were a memory allocation failiure
 */
NameListAttribute_t* newAttribute(const char* name);

/* Function: newCompoundAttribute
 * Description: Allocates a new compound attribute definition.
 * Input:
 *	 name: Name of the compound attribute, may be NULL, then the attribute will
 *	       not be given any name
 * Returns: The created compound attribute
 */
CompoundAttributeDef_t* newCompoundAttribute(const char* name);

/* Function: createCompoundFromType
 * Description: Will transfer all attributes that are common between
 *              the NameListType_t object and CompoundAttributeDef_t object
 *		to a new compound attribute, and then give it a name.
 * Input:
 *	 inType: The type to get all common attribute definitions from
 *	 name:   The name of the compound attribute definition
 * Returns: The Copied attribute
 */
CompoundAttributeDef_t* createCompoundFromType(NameListType_t* inType, char* name);

/* Function: addCompoundAttributeToType
 * Description: Adds the compound attribute def to the given name list type
 *		Note that the responsibility for deleting this object
 *		has been given the NameListType object
 * Returns: >= 0 if all was ok
 *	    -1 if something went wrong, in this case responsibility has not been taken
 */
herr_t addCompoundAttributeToType(NameListType_t* newType,CompoundAttributeDef_t* compoundAttr);

/* Function: addAttributeToGroup
 * Description: Appends an attribute to the specified group
 */
herr_t addAttributeToGroup(NameListGroup_t* group, NameListAttribute_t* attr);

/* Function: addAttributeToDataset
 * Description: Basically the same as 'addAttributeToGroup', but the attribute
 *		is appended to the dataset instead
 */
herr_t addAttributeToDataset(NameListDataset_t* dset, NameListAttribute_t* attr);

/* Function: freeCompoundAttribute
 * Description: Deallocates the compound attribute
 */
void freeCompoundAttribute(CompoundAttributeDef_t* attr);

/* Function: freeAttribute
 * Description: Deallocates the attribute
 */
void freeAttribute(NameListAttribute_t* attr);

/* Function: freeNameListType
 * Description: Deallocates the type
 * Note: A good idea is to always delete the types last, since
 *       references to them can be residing in the groups.
 */
void freeNameListType(NameListType_t* type);

/* Function: freeInternalDataset
 * Description: Deallocates the internals of the dataset
 * Note! This will also delete the NameListType, if the isOwnedByGroup
 *       NameListType_t.isOwnedByGroup == 0.
 */
void freeInternalDataset(NameListDataset_t* dset);

/* Function: freeDataset
 * Description: Deallocates both the internals and also the actual object
 */
void freeDataset(NameListDataset_t* dset);

/* Function: addTypeToLocalGroup
 * Description: Adds the specified type to the local list in the group
 * Input:
 *	 group: The group that should get the type appended
 *	 type:  The type that should be appended to the group,
 *              this will not be copied, so be sure that
 *              when doing this, the responsibility is taken by
 *              the group to delete this memory
 * Returns: >=0 if everything was ok
 *          -1  if something went wrong, then the responsibility has not
 *              been taken.
 */
herr_t addTypeToLocalGroup(NameListGroup_t* group, NameListType_t* type);

/* Function: addTypeToGlobalGroup
 * Description: Basically the same as for the 'addTypeToLocalGroup' with
 *              the exception that this is done for the list of global types
 */
herr_t addTypeToGlobalGroup(NameListGroup_t* group, NameListType_t* type);

/* Function: doesTypeExistInGlobalGroup
 * Description: Searches the global type list if any
 *              occurence of the given objno exists among the
 *              types
 * Input:
 *	 grp: The group to check
 *       objno: A pointer to a list of two objno:s i.e. unsigned long objno[2]
 * Returns: The index number in the global list if one occurence was found
 *          -1 If no occurence could be found.
 */
int doesTypeExistInGlobalGroup(NameListGroup_t* grp,unsigned long* objno);

/* Function: doesTypeExistInLocalGroup
 * Description: Searches the local type list for any occurences
 *              of objno.
 * See the description of doesTypeExistInGlobalGroup
 */
int doesTypeExistInLocalGroup(NameListGroup_t* grp,unsigned long* objno);

/* Function: removeTypeFromLocalGroup
 * Description: Removes the type with a matching objno from the list of local types
 *              and returns the name list type.
 *		Note that the responsibility for deleting this object
 *		May still be up the group.
 *		Since it might be a pointer to a group in the global type defs
 * Returns: The object with a matching objno
 */
NameListType_t* removeTypeFromLocalGroup(NameListGroup_t* group,unsigned long* objno);

/* Function: removeTypeFromGlobalGroup
 * Description: Same as for removeTypeFromLocalGroup and also here
 *              be careful when deleting the returned memory
 */
NameListType_t* removeTypeFromGlobalGroup(NameListGroup_t* group,unsigned long* objno);

/* Function:	whatSizeIsHdfFormat
 * Description:	Calculates the size in bytes that the specified type takes.
 * Input:
 *		format, can be one of 'int','float',... (see hlhdf.h) for
 *		more types.
 * Returns:	-1 if error, otherwise the size in bytes
 */
int whatSizeIsHdfFormat(const char* format);

/* Function:	isFormatSupported
 * Description:	Checks if the given format is supported.
 * Input:
 *		format, can be one of 'int','float',... (see hlhdf.h) for
 *		more types.
 * Returns:	1 if it is supported, otherwise 0.
 */
int isFormatSupported(const char* format);

#endif
