/* --------------------------------------------------------------------------
$Id: vhlhdf.h,v 1.1.1.1 2001/12/14 12:02:40 ahenja Exp $
Program:        

Description:    Common functions for working with the HighLevel
		protocol.

Author(s):      Anders Henja

Copyright:	Swedish Meteorological and Hydrological Institute, 2000

History:
$Log: vhlhdf.h,v $
Revision 1.1.1.1  2001/12/14 12:02:40  ahenja
First CVS version of hlhdf

Revision 1.8  2001/11/09 08:31:53  ahenja
Updated with possibility to write references, unfortunately i havent got the time to implement read support.

Revision 1.7  2001/11/08 12:26:37  ahenja
Updated.

 * Revision 1.6  2001/01/18  12:39:32  ahenja
 * *** empty log message ***
 *
Revision 1.5  2000/03/22 08:39:38  ahenja
Updated with spell checking and header information.

-----------------------------------------------------------------------------*/
#ifndef HL_H
#define HL_H

#include "hlhdf.h"

#define DEFAULT_SIZE_NODELIST 20
#define DATAFORMAT_UNDEFINED "UNDEFINED"

typedef enum HL_Type {UNDEFINED_ID=-1, ATTRIBUTE_ID=0, GROUP_ID,
			  DATASET_ID, TYPE_ID, REFERENCE_ID} HL_Type;
typedef enum HL_DataType {DTYPE_UNDEFINED_ID=-1,HL_SIMPLE=0,HL_ARRAY} HL_DataType;

typedef enum HL_NodeMark {NMARK_ORIGINAL=0, NMARK_CREATED, NMARK_CHANGED, NMARK_SELECT} HL_NodeMark;


typedef struct {
   char attrname[256];
   size_t offset;
   size_t size;
   char format[256];
   int ndims;
   size_t dims[4];
} HL_CompoundTypeAttribute;

typedef struct {
   char typename[256];
   unsigned long objno[2];
   size_t size;
   int nAttrs;
   int nAllocAttrs;
   HL_CompoundTypeAttribute** attrs;
} HL_CompoundTypeDescription;

typedef struct HL_Node {
   HL_Type type;
   char name[256];
   int ndims;
   hsize_t dims[4];
   unsigned char* data;
   unsigned char* rawdata;/* Ahe added this 2001-01-17 */
   char format[64];
   hid_t typeId;
   size_t dSize; /* Size for data (fixed type) */
   size_t rdSize; /* Size for rawdata */
   HL_DataType dataType;
   hid_t hdfId;
   HL_NodeMark mark;
   HL_CompoundTypeDescription* compoundDescription;
} HL_Node;

typedef struct {
   char filename[256];
   char tmp_name[512]; /*Hopefully the name will not be longer than this*/
   int nNodes;
   int nAllocNodes;
   HL_Node** nodes;
} HL_NodeList;

/* Function: newHL_Node
 * Description: Creates an empty HL node of undefined type
 *
 * Returns: The node if everything was ok, otherwise NULL
 */
HL_Node* newHL_Node(const char* name);

/* Function: newHL_NodeList
 * Description: Creates an empty HL node list.
 *
 * Returns: The nodelist if everything was ok, otherwise NULL
 */
HL_NodeList* newHL_NodeList();

/* Function: freeHL_Node
 * Description: Frees one node.
 *
 * Returns: Nothing
 */
void freeHL_Node(HL_Node* node);

/* Function: freeHL_NodeList
 * Description: Frees a complete Node list including the nodes.
 *
 * Returns: Nothing
 */
void freeHL_NodeList(HL_NodeList* nodelist);

/* Function: newHL_Group
 * Description: Creates an empty HL node of GROUP_ID type.
 *
 * Returns: The node if everything was ok, otherwise NULL
 */
HL_Node* newHL_Group(const char* name);

/* Function: newHL_Attribute
 * Description: Creates an empty HL node of ATTRIBUTE_ID type.
 *
 * Returns: The node if everything was ok, otherwise NULL
 */
HL_Node* newHL_Attribute(const char* name);

/* Function: newHL_Dataset
 * Description: Creates an empty dataset node of DATASET_ID type.
 *
 * Returns: The node if everything was ok, otherwise NULL
 */
HL_Node* newHL_Dataset(const char* name);

/* Function: newHL_Datatype
 * Description: Creates an HL node of TYPE_ID type
 *
 * Returns: The node if everything was ok, otherwise NULL
 */
HL_Node* newHL_Datatype(const char* name);

/* Function: newHL_Reference
 * Description: Creates an HL node of REFERENCE_ID type
 *
 * Returns: The node if everything was ok, otherwise NULL
 */
HL_Node* newHL_Reference(const char* name);

/* Function: newHL_CompoundTypeAttribute
 * Description: Creates a compound attribute member node.
 * Returns: The compound member if everything was ok, otherwise NULL
 */
HL_CompoundTypeAttribute* newHL_CompoundTypeAttribute(char* attrname,
						      size_t offset,
						      char* format,
						      size_t size,
						      int ndims,
						      size_t* dims);
/* Function: newHL_CompoundTypeDescription
 * Description: Creates a compound type description list.
 * Returns: The compound type description if everything was ok, otherwise NULL.
 */
HL_CompoundTypeDescription* newHL_CompoundTypeDescription();

/* Function: freeHL_CompoundTypeAttribute
 * Description: Frees one compound type attribute.
 * Returns: Nothing
 */
void freeHL_CompoundTypeAttribute(HL_CompoundTypeAttribute* attr);

/* Function: freeHL_CompoundTypeDescription
 * Description: Frees one compound type, including all members
 * Returns: Nothing
 */
void freeHL_CompoundTypeDescription(HL_CompoundTypeDescription* typelist);

/* Function: copyHL_Node
 * Description: Copies one HL_Node.
 *
 * Returns: The node if ok, otherwise NULL
 */
HL_Node* copyHL_Node(HL_Node* node);

/* Function: copyCompoundTypeDescription
 * Description: Copies one compound type description
 *
 * Returns: The type description if ok, otherwise NULL
 */
HL_CompoundTypeDescription* copyCompoundTypeDescription(HL_CompoundTypeDescription* descr);

/* Function: addNode
 * Description: Adds a Node to the end of the nodelist.
 *
 * Returns: 1 if everything went ok, otherwise 0.
 */
int addNode(HL_NodeList* nodelist, HL_Node* node);

/* Function: getNode
 * Description: Locates a node called nodeName in the nodelist, then
 *		returns a reference to this node, i.e. do not delete it.
 *
 * Returns: The node if it could be found, otherwise NULL.
 */
HL_Node* getNode(HL_NodeList* nodelist,const char* nodeName);

/* Function: setScalarValue
 * Description: Sets a scalar value in the specified node.
 *
 * Returns: 1 if everything was ok, otherwise 0.
 */
int setScalarValue(HL_Node* node,size_t sz,unsigned char* value,const char* fmt,hid_t typid);

/* Function: setArrayValue
 * Description: Sets an array value in the specified node.
 *
 * Returns: 1 if everything was ok, otherwise 0.
 */
int setArrayValue(HL_Node* node,size_t sz,int ndims,hsize_t* dims,unsigned char* value,
		  const char* fmt,hid_t typid);

/* Function: mark_all_nodes
 *
 * Description: Sets all the nodes in the nodelist to
 *		selected node mark
 *
 * Returns: N/A
 */
void markAllNodes(HL_NodeList* nodelist, HL_NodeMark mark);

/* Function: extractParentChildName
 * Description: Extracts the parent and child name from the node name.
 *		Example:
 *		The nodes name is '/group1/group2/group3',
 *		The extractParentChildName is called, then
 *		parent will be '/group1/group2' and child will be 'group3'.
 *
 * Returns: 1 if everything was ok, otherwise 0.
 */
int extractParentChildName(HL_Node* node, char* parent, char* child);

/* Function: commitDatatype
 * Description: Will mark the node to be committed, only
 *		applicable on nodes of type 'TYPE_ID'.
 * Returns: 1 if everything was ok, otherwise 0.
 */
int commitDatatype(HL_Node* node,hid_t testStruct_hid);

/* Function: scanNodeList
 * Description: Just prints the names in the nodelist.
 * Returns: Nothing
 */
void scanNodeList(HL_NodeList* nodelist);

/* Function: findCompoundTypeDescription
 * Description: Searches the nodelist for any type node, that has got the
 *		same object id as objno0 and objno1.
 * Returns: The compound type description if found, otherwise NULL.
 */
HL_CompoundTypeDescription* findCompoundTypeDescription(HL_NodeList* nodelist,
							    unsigned long objno0,
							    unsigned long objno1);

/* Function: scanCompoundTypeDescription
 * Description: Prints all members in an compound type description.
 * Returns: Nothing
 */
void scanCompoundTypeDescription(HL_CompoundTypeDescription* typelist);

#endif
