/* --------------------------------------------------------------------------
$Id: write_hlhdf.h,v 1.1.1.1 2001/12/14 12:02:39 ahenja Exp $
Program:        

Description:    Common functions for writing an HDF5 file.

Author(s):      Anders Henja

Copyright:	Swedish Meteorological and Hydrological Institute, 2000

History:
$Log: write_hlhdf.h,v $
Revision 1.1.1.1  2001/12/14 12:02:39  ahenja
First CVS version of hlhdf

Revision 1.4  2001/12/07 13:20:32  ahenja
Updated with possibilities to read and write references.

Revision 1.3  2001/11/09 08:30:56  ahenja
Updated with possibility to write references, unfortunately i havent got the time to implement read support.

Revision 1.2  2000/03/22 08:41:33  ahenja
Updated with spell checking and header information.

-----------------------------------------------------------------------------*/
#ifndef WRITE_HLHDF_H
#define WRITE_HLHDF_H
#include "hlhdf.h"

/* Function:	commitType
 * Description: Turns a self defined type into a named type, i.e.
 *		gives it a name.
 * Input:
 *		loc_id  In what group or file, this type this should
 *			be placed.
 *		name	The name of this type
 *		type_id	The reference to the self defined type. 
 * Returns:	<0 is failure, otherwise ok.
 */
herr_t commitType(hid_t loc_id, const char* name, hid_t type_id);

/* Function:	createStringType
 * Description:	Creates an HDF5 type, for a string of length 'length'
 * Input:
 *		length	The length of the string
 * Returns:	The reference to the created type, -1 if failiure
 */
hid_t createStringType(size_t length);

/* Function:	setTypeSize
 * Description:	Changes the size of the given type to be of length
 *		'theSize'
 * Input:
 *		type_id	The type that should have the size changed.
 *		theSize	How big the specified type should have
 * Returns:	<0 is failiure, otherwise ok.
 */
herr_t setTypeSize(hid_t type_id,size_t theSize);

/* Function:	closeType
 * Description:	Just encapsulates H5Tclose, it is used for releasing memory held
 *		by the specified type.
 * Input:
 *		type_id	The type that should be released.
 * Returns:	<0 is failure, otherwise ok.
 */
herr_t closeType(hid_t type_id);

/* Function:	writeScalarDataAttribute
 * Description:	Writes a scalar attribute to the file.
 * Input:
 *		loc_id 	In what group or file or dataset this attribute should
 *		       	be written.
 *		type_id	The type of what should be written
 *		name	The name of the attribute
 *		buf	A pointer to the data
 * Returns:	<0 is failiure, otherwise ok.
 */
herr_t writeScalarDataAttribute(hid_t loc_id, hid_t type_id, const char* name, void* buf);

/* Function:	writeScalarDataAttribute_fmt
 * Description:	Same as writeScalarDataAttribute, but instead of a type as hid_t,
 *		the format can be specified as a string.
 *		NOTE! If a selfdefined type should be written, don't use this
 *		function.
 * Input:
 *		fmt	Can be one of:
 *			'char', 'schar', 'uchar', 'short', 'ushort',
 *			'int', 'uint', 'long', 'ulong', 'llong',
 *			'ullong', 'float', 'double', 'hsize', 'hssize',
 *			'herr', 'hbool'
 * Returns:	<0 is failiure, otherwise ok.
 */
herr_t writeScalarDataAttribute_fmt(hid_t loc_id, const char* fmt, const char* name, void* buf);

/* Function:	writeSimpleDataAttribute
 * Description:	Writes a ndims-dimensional array to disk with the dimensions as specified
 *		in dims[0] .. dims[ndims-1]
 * Input:
 *		loc_id	The location, the data should be written.
 *		type_id	The type of the data that should be written
 *		name	The name of the attribute
 *		ndims	The rank of the data
 *		dims	The dimensions of the data
 *		buf	The data
 * Returns:	<0 is failure, otherwise ok.
 */
herr_t writeSimpleDataAttribute(hid_t loc_id, hid_t type_id, const char* name, int ndims, hsize_t* dims, void* buf);

/* Function:	writeSimpleDataAttribute_fmt
 * Description:	Same as writeSimpleDataAttribute, but instead of a type as hid_t, the
 *		format can be specified as a string.
 *		NOTE! If a selfdefined type should be written, don't use this
 *		function.
 * Input:
 *		fmt	Can be one of:
 *			'char', 'schar', 'uchar', 'short', 'ushort',
 *			'int', 'uint', 'long', 'ulong', 'llong',
 *			'ullong', 'float', 'double', 'hsize', 'hssize',
 *			'herr', 'hbool'
 * Returns:	<0 is failure, otherwise ok.
 */
herr_t writeSimpleDataAttribute_fmt(hid_t loc_id, const char* fmt,
				    const char* name, int ndims, hsize_t* dims, void* buf);

/* Function:	createSimpleDataset
 * Description:	Creates a simple dataset and if buf != NULL, the dataset will get
 *		the data filled in.
 * Input:
 *		loc_id	The location the dataset should be created in
 *		type_id	The type of the data
 *		ndims	The rank of the data
 *		dims	The dimensions of the data
 *		buf	The data
 *		compress	The compression level that should be used,
 *				0 means no compression
 *				9 means the best compression
 *				Note: The Higher compression level, the longer it takes.
 * Returns:	<0 is failure, otherwise a reference to the dataset.
 */
hid_t createSimpleDataset(hid_t loc_id, hid_t type_id, const char* name, int ndims, hsize_t* dims, void* buf, int compress);

/* Function:	createSimpleDataset_fmt
 * Description:	Same as createSimpleDataset, but instead of a type as hid_t, the
 *		format can be specified as a string.
 *		NOTE! If a selfdefined type should be written, don't use this
 *		function.
 * Input:
 *		fmt	Can be one of:
 *			'char', 'schar', 'uchar', 'short', 'ushort',
 *			'int', 'uint', 'long', 'ulong', 'llong',
 *			'ullong', 'float', 'double', 'hsize', 'hssize',
 *			'herr', 'hbool'
 * Returns:	<0 is failure, otherwise a reference to the dataset.
 */
hid_t createSimpleDataset_fmt(hid_t loc_id, const char* fmt, const char* name, int ndims, hsize_t* dims, void* buf,int compress);

/* Function:	closeDataset
 * Description:	Closes the reference to the dataset
 * Input:
 *		loc_id	The dataset reference that should be closed.
 * Returns:	<0 if error, otherwise ok
 */
herr_t closeDataset(hid_t loc_id);

/* Function:	createCompoundType
 * Description:	Creates a new compound type
 * Input:
 *		size	The size of the compound type
 * Returns:	<0 if failure, otherwise a reference to the created type
 */
hid_t createCompoundType(size_t size);

/* Function:	addAttributeToCompoundType
 * Description:	Adds one attribute to a compound type
 * Input:
 *		loc_id	The type the attribute should be inserted in.
 *		name	The name of the member attribute
 *		offset	Offset of this attribute
 *		type_id	The type of the attribute
 * Returns:	<0 if error, otherwise ok.
 */
herr_t addAttributeToCompoundType(hid_t loc_id, const char* name, size_t offset,hid_t type_id);

/* Function:	addAttributeToCompoundType_fmt
 * Description:	Same as addAttributeToCompoundType, but instead of a type as hid_t, the
 *		format can be specified as a string.
 * Input:
 *		fmt	Can be one of:
 *			'char', 'schar', 'uchar', 'short', 'ushort',
 *			'int', 'uint', 'long', 'ulong', 'llong',
 *			'ullong', 'float', 'double', 'hsize', 'hssize',
 *			'herr', 'hbool'
 * Returns:	<0 is failure, otherwise ok.
 */
herr_t addAttributeToCompoundType_fmt(hid_t loc_id, const char* name, size_t offset,const char* fmt);

/* Function:	addArrayToCompoundType
 * Description:	Adds an array attribute to a compound type
 * Input:
 *		loc_id	The type this attribute should be inserted in
 *		name	The name of the attribute
 *		offset	Offset of this attribute
 *		ndims	The rank of the array attribute (max 4)
 *		dims	The dimension of the array attribute
 *		type_id	The type of the array attribute
 * Returns:	<0 is failure, otherwise ok.
 */
herr_t addArrayToCompoundType(hid_t loc_id, const char* name, size_t offset, int ndims,size_t* dims,hid_t type_id);

/* Function:	addArrayToCompoundType_fmt
 * Description:	Same as addArrayToCompoundType, but instead of a type as hid_t, the format
 *		can be specified as a string.
 * Input:
 *		fmt	Can be one of:
 *			'char', 'schar', 'uchar', 'short', 'ushort',
 *			'int', 'uint', 'long', 'ulong', 'llong',
 *			'ullong', 'float', 'double', 'hsize', 'hssize',
 *			'herr', 'hbool'
 * Returns:	<0 is failure, otherwise ok.
 */
herr_t addArrayToCompoundType_fmt(hid_t loc_id, const char* name, size_t offset, \
				  int ndims, size_t* dims, const char* fmt);

/* Function:	createGroup
 * Description:	Creates a new group connected to a file member.
 * Input:
 *		loc_id	The location this group should be connected to,
 *			can be either a file or group.
 *		groupname
 *			The name of the group
 *		comment	The comment of this group.
 * Returns:	<0 is failiure, otherwise a reference to the created group
 */
hid_t createGroup(hid_t loc_id, const char* groupname,const char* comment);

/* Function:	closeGroup
 * Description:	Closes a group
 * Input:
 *		loc_id	The group reference that should be closed.
 * Returns:	<0 if error, otherwise ok.
 */
herr_t closeGroup(hid_t loc_id);

/* Function:	createReference
 * Description:	Creates a reference from loc_id, name to
 *		the targetname.
 * Input:
 *		loc_id	The place the reference should be placed
 *		file_id	The file id.
 *		name	The name of the reference
 *		targetname	Where the reference points.
 * Returns:	<0 if error, otherwise ok.
 */
herr_t createReference(hid_t loc_id, const char* name,
		       const char* targetname);

#endif
